@@ -18,14 +18,13 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
-	"knative.dev/client/pkg/kn/flags"
+	knflags "knative.dev/client/pkg/kn/flags"
 	servinglib "knative.dev/client/pkg/serving"
 	"knative.dev/client/pkg/util"
 	"knative.dev/serving/pkg/apis/serving"
@@ -33,68 +32,35 @@ import (
 )
 
 type ConfigurationEditFlags struct {
+	//Fields for PodSpecFlags
+	PodSpecFlags knflags.PodSpecFlags
+
 	// Direct field manipulation
-	Image   uniqueStringArg
-	Env     []string
-	EnvFrom []string
-	Mount   []string
-	Volume  []string
-
-	Command string
-	Arg     []string
-
-	RequestsFlags, LimitsFlags ResourceFlags
-	MinScale                   int
-	MaxScale                   int
-	ConcurrencyTarget          int
-	ConcurrencyLimit           int
-	ConcurrencyUtilization     int
-	AutoscaleWindow            string
-	Port                       int32
-	Labels                     []string
-	LabelsService              []string
-	LabelsRevision             []string
-	NamePrefix                 string
-	RevisionName               string
-	ServiceAccountName         string
-	ImagePullSecrets           string
-	Annotations                []string
-	ClusterLocal               bool
-	User                       int64
+	Scale                  int
+	MinScale               int
+	MaxScale               int
+	ConcurrencyTarget      int
+	ConcurrencyLimit       int
+	ConcurrencyUtilization int
+	AutoscaleWindow        string
+	Labels                 []string
+	LabelsService          []string
+	LabelsRevision         []string
+	RevisionName           string
+	Annotations            []string
+	ClusterLocal           bool
 
 	// Preferences about how to do the action.
 	LockToDigest         bool
 	GenerateRevisionName bool
 	ForceCreate          bool
 
+	Filename string
+
 	// Bookkeeping
 	flags []string
 }
 
-type ResourceFlags struct {
-	CPU    string
-	Memory string
-}
-
-// -- uniqueStringArg Value
-// Custom implementation of flag.Value interface to prevent multiple value assignment.
-// Useful to enforce unique use of flags, e.g. --image.
-type uniqueStringArg string
-
-func (s *uniqueStringArg) Set(val string) error {
-	if len(*s) > 0 {
-		return errors.New("can be provided only once")
-	}
-	*s = uniqueStringArg(val)
-	return nil
-}
-
-func (s *uniqueStringArg) Type() string {
-	return "string"
-}
-
-func (s *uniqueStringArg) String() string { return string(*s) }
-
 // markFlagMakesRevision indicates that a flag will create a new revision if you
 // set it.
 func (p *ConfigurationEditFlags) markFlagMakesRevision(f string) {
@@ -103,70 +69,32 @@ func (p *ConfigurationEditFlags) markFlagMakesRevision(f string) {
 
 // addSharedFlags adds the flags common between create & update.
 func (p *ConfigurationEditFlags) addSharedFlags(command *cobra.Command) {
-	command.Flags().VarP(&p.Image, "image", "", "Image to run.")
-	p.markFlagMakesRevision("image")
-	command.Flags().StringArrayVarP(&p.Env, "env", "e", []string{},
-		"Environment variable to set. NAME=value; you may provide this flag "+
-			"any number of times to set multiple environment variables. "+
-			"To unset, specify the environment variable name followed by a \"-\" (e.g., NAME-).")
-	p.markFlagMakesRevision("env")
-
-	command.Flags().StringArrayVarP(&p.EnvFrom, "env-from", "", []string{},
-		"Add environment variables from a ConfigMap (prefix cm: or config-map:) or a Secret (prefix secret:). "+
-			"Example: --env-from cm:myconfigmap or --env-from secret:mysecret. "+
-			"You can use this flag multiple times. "+
-			"To unset a ConfigMap/Secret reference, append \"-\" to the name, e.g. --env-from cm:myconfigmap-.")
-	p.markFlagMakesRevision("env-from")
-
-	command.Flags().StringArrayVarP(&p.Mount, "mount", "", []string{},
-		"Mount a ConfigMap (prefix cm: or config-map:), a Secret (prefix secret: or sc:), or an existing Volume (without any prefix) on the specified directory. "+
-			"Example: --mount /mydir=cm:myconfigmap, --mount /mydir=secret:mysecret, or --mount /mydir=myvolume. "+
-			"When a configmap or a secret is specified, a corresponding volume is automatically generated. "+
-			"You can use this flag multiple times. "+
-			"For unmounting a directory, append \"-\", e.g. --mount /mydir-, which also removes any auto-generated volume.")
-	p.markFlagMakesRevision("mount")
-
-	command.Flags().StringArrayVarP(&p.Volume, "volume", "", []string{},
-		"Add a volume from a ConfigMap (prefix cm: or config-map:) or a Secret (prefix secret: or sc:). "+
-			"Example: --volume myvolume=cm:myconfigmap or --volume myvolume=secret:mysecret. "+
-			"You can use this flag multiple times. "+
-			"To unset a ConfigMap/Secret reference, append \"-\" to the name, e.g. --volume myvolume-.")
-	p.markFlagMakesRevision("volume")
-
-	command.Flags().StringVarP(&p.Command, "cmd", "", "",
-		"Specify command to be used as entrypoint instead of default one. "+
-			"Example: --cmd /app/start or --cmd /app/start --arg myArg to pass aditional arguments.")
-	p.markFlagMakesRevision("cmd")
-
-	command.Flags().StringArrayVarP(&p.Arg, "arg", "", []string{},
-		"Add argument to the container command. "+
-			"Example: --arg myArg1 --arg --myArg2 --arg myArg3=3. "+
-			"You can use this flag multiple times.")
-	p.markFlagMakesRevision("arg")
-
-	command.Flags().StringVar(&p.RequestsFlags.CPU, "requests-cpu", "", "The requested CPU (e.g., 250m).")
-	p.markFlagMakesRevision("requests-cpu")
-
-	command.Flags().StringVar(&p.RequestsFlags.Memory, "requests-memory", "", "The requested memory (e.g., 64Mi).")
-	p.markFlagMakesRevision("requests-memory")
-
-	command.Flags().StringVar(&p.LimitsFlags.CPU, "limits-cpu", "", "The limits on the requested CPU (e.g., 1000m).")
-	p.markFlagMakesRevision("limits-cpu")
-
-	command.Flags().StringVar(&p.LimitsFlags.Memory, "limits-memory", "",
-		"The limits on the requested memory (e.g., 1024Mi).")
-	p.markFlagMakesRevision("limits-memory")
+	flagNames := p.PodSpecFlags.AddFlags(command.Flags())
+	for _, name := range flagNames {
+		p.markFlagMakesRevision(name)
+	}
 
 	command.Flags().IntVar(&p.MinScale, "min-scale", 0, "Minimal number of replicas.")
+	command.Flags().MarkHidden("min-scale")
 	p.markFlagMakesRevision("min-scale")
 
 	command.Flags().IntVar(&p.MaxScale, "max-scale", 0, "Maximal number of replicas.")
+	command.Flags().MarkHidden("max-scale")
 	p.markFlagMakesRevision("max-scale")
 
+	command.Flags().IntVar(&p.Scale, "scale", 0, "Minimum and maximum number of replicas.")
+	p.markFlagMakesRevision("scale")
+
+	command.Flags().IntVar(&p.MinScale, "scale-min", 0, "Minimum number of replicas.")
+	p.markFlagMakesRevision("scale-min")
+
+	command.Flags().IntVar(&p.MaxScale, "scale-max", 0, "Maximum number of replicas.")
+	p.markFlagMakesRevision("scale-max")
+
 	command.Flags().StringVar(&p.AutoscaleWindow, "autoscale-window", "", "Duration to look back for making auto-scaling decisions. The service is scaled to zero if no request was received in during that time. (eg: 10s)")
 	p.markFlagMakesRevision("autoscale-window")
 
-	flags.AddBothBoolFlagsUnhidden(command.Flags(), &p.ClusterLocal, "cluster-local", "", false,
+	knflags.AddBothBoolFlagsUnhidden(command.Flags(), &p.ClusterLocal, "cluster-local", "", false,
 		"Specify that the service be private. (--no-cluster-local will make the service publicly available)")
 	//TODO: Need to also not change revision when already set (solution to issue #646)
 	p.markFlagMakesRevision("cluster-local")
@@ -185,9 +113,6 @@ func (p *ConfigurationEditFlags) addSharedFlags(command *cobra.Command) {
 		"Percentage of concurrent requests utilization before scaling up.")
 	p.markFlagMakesRevision("concurrency-utilization")
 
-	command.Flags().Int32VarP(&p.Port, "port", "p", 0, "The port where application listens on.")
-	p.markFlagMakesRevision("port")
-
 	command.Flags().StringArrayVarP(&p.Labels, "label", "l", []string{},
 		"Labels to set for both Service and Revision. name=value; you may provide this flag "+
 			"any number of times to set multiple labels. "+
@@ -214,30 +139,16 @@ func (p *ConfigurationEditFlags) addSharedFlags(command *cobra.Command) {
 			"{{.Generation}} for the generation, and {{.Random [n]}} for n random consonants.")
 	p.markFlagMakesRevision("revision-name")
 
-	flags.AddBothBoolFlagsUnhidden(command.Flags(), &p.LockToDigest, "lock-to-digest", "", true,
+	knflags.AddBothBoolFlagsUnhidden(command.Flags(), &p.LockToDigest, "lock-to-digest", "", true,
 		"Keep the running image for the service constant when not explicitly specifying "+
 			"the image. (--no-lock-to-digest pulls the image tag afresh with each new revision)")
 	// Don't mark as changing the revision.
 
-	command.Flags().StringVar(&p.ServiceAccountName,
-		"service-account",
-		"",
-		"Service account name to set. An empty argument (\"\") clears the service account. The referenced service account must exist in the service's namespace.")
-	p.markFlagMakesRevision("service-account")
-
 	command.Flags().StringArrayVarP(&p.Annotations, "annotation", "a", []string{},
 		"Service annotation to set. name=value; you may provide this flag "+
 			"any number of times to set multiple annotations. "+
 			"To unset, specify the annotation name followed by a \"-\" (e.g., name-).")
 	p.markFlagMakesRevision("annotation")
-
-	command.Flags().StringVar(&p.ImagePullSecrets,
-		"pull-secret",
-		"",
-		"Image pull secret to set. An empty argument (\"\") clears the pull secret. The referenced secret must exist in the service's namespace.")
-	p.markFlagMakesRevision("pull-secret")
-	command.Flags().Int64VarP(&p.User, "user", "", 0, "The user ID to run the container (e.g., 1001).")
-	p.markFlagMakesRevision("user")
 }
 
 // AddUpdateFlags adds the flags specific to update.
@@ -250,7 +161,11 @@ func (p *ConfigurationEditFlags) AddCreateFlags(command *cobra.Command) {
 	p.addSharedFlags(command)
 	command.Flags().BoolVar(&p.ForceCreate, "force", false,
 		"Create service forcefully, replaces existing service if any.")
-	command.MarkFlagRequired("image")
+	command.Flags().StringVarP(&p.Filename, "filename", "f", "", "Create a service from file. "+
+		"The created service can be further modified by combining with other options. "+
+		"For example, -f /path/to/file --env NAME=value adds also an environment variable.")
+	command.MarkFlagFilename("filename")
+	p.markFlagMakesRevision("filename")
 }
 
 // Apply mutates the given service according to the flags in the command.
@@ -261,9 +176,9 @@ func (p *ConfigurationEditFlags) Apply(
 
 	template := &service.Spec.Template
 	if cmd.Flags().Changed("env") {
-		envMap, err := util.MapFromArrayAllowingSingles(p.Env, "=")
+		envMap, err := util.MapFromArrayAllowingSingles(p.PodSpecFlags.Env, "=")
 		if err != nil {
-			return errors.Wrap(err, "Invalid --env")
+			return fmt.Errorf("Invalid --env: %w", err)
 		}
 
 		envToRemove := util.ParseMinusSuffix(envMap)
@@ -276,7 +191,7 @@ func (p *ConfigurationEditFlags) Apply(
 	if cmd.Flags().Changed("env-from") {
 		envFromSourceToUpdate := []string{}
 		envFromSourceToRemove := []string{}
-		for _, name := range p.EnvFrom {
+		for _, name := range p.PodSpecFlags.EnvFrom {
 			if name == "-" {
 				return fmt.Errorf("\"-\" is not a valid value for \"--env-from\"")
 			} else if strings.HasSuffix(name, "-") {
@@ -293,14 +208,14 @@ func (p *ConfigurationEditFlags) Apply(
 	}
 
 	if cmd.Flags().Changed("mount") || cmd.Flags().Changed("volume") {
-		mountsToUpdate, mountsToRemove, err := util.OrderedMapAndRemovalListFromArray(p.Mount, "=")
+		mountsToUpdate, mountsToRemove, err := util.OrderedMapAndRemovalListFromArray(p.PodSpecFlags.Mount, "=")
 		if err != nil {
-			return errors.Wrap(err, "Invalid --mount")
+			return fmt.Errorf("Invalid --mount: %w", err)
 		}
 
-		volumesToUpdate, volumesToRemove, err := util.OrderedMapAndRemovalListFromArray(p.Volume, "=")
+		volumesToUpdate, volumesToRemove, err := util.OrderedMapAndRemovalListFromArray(p.PodSpecFlags.Volume, "=")
 		if err != nil {
-			return errors.Wrap(err, "Invalid --volume")
+			return fmt.Errorf("Invalid --volume: %w", err)
 		}
 
 		err = servinglib.UpdateVolumeMountsAndVolumes(template, mountsToUpdate, mountsToRemove, volumesToUpdate, volumesToRemove)
@@ -320,7 +235,7 @@ func (p *ConfigurationEditFlags) Apply(
 
 	imageSet := false
 	if cmd.Flags().Changed("image") {
-		err = servinglib.UpdateImage(template, p.Image.String())
+		err = servinglib.UpdateImage(template, p.PodSpecFlags.Image.String())
 		if err != nil {
 			return err
 		}
@@ -340,54 +255,97 @@ func (p *ConfigurationEditFlags) Apply(
 		servinglib.UnsetUserImageAnnot(template)
 	}
 
-	limitsResources, err := p.computeResources(p.LimitsFlags)
+	if cmd.Flags().Changed("limits-cpu") || cmd.Flags().Changed("limits-memory") {
+		if cmd.Flags().Changed("limit") {
+			return fmt.Errorf("only one of (DEPRECATED) --limits-cpu / --limits-memory and --limit can be specified")
+		}
+		fmt.Fprintf(cmd.OutOrStdout(), "\nWARNING: flags --limits-cpu / --limits-memory are deprecated and going to be removed in future release, please use --limit instead.\n\n")
+	}
+
+	if cmd.Flags().Changed("requests-cpu") || cmd.Flags().Changed("requests-memory") {
+		if cmd.Flags().Changed("request") {
+			return fmt.Errorf("only one of (DEPRECATED) --requests-cpu / --requests-memory and --request can be specified")
+		}
+		fmt.Fprintf(cmd.OutOrStdout(), "\nWARNING: flags --requests-cpu / --requests-memory are deprecated and going to be removed in future release, please use --request instead.\n\n")
+	}
+
+	limitsResources, err := p.computeResources(p.PodSpecFlags.LimitsFlags)
 	if err != nil {
 		return err
 	}
-	requestsResources, err := p.computeResources(p.RequestsFlags)
+	requestsResources, err := p.computeResources(p.PodSpecFlags.RequestsFlags)
 	if err != nil {
 		return err
 	}
-	err = servinglib.UpdateResources(template, requestsResources, limitsResources)
+	err = servinglib.UpdateResourcesDeprecated(template, requestsResources, limitsResources)
+	if err != nil {
+		return err
+	}
+
+	requestsToRemove, limitsToRemove, err := p.PodSpecFlags.Resources.Validate()
+	if err != nil {
+		return err
+	}
+
+	err = servinglib.UpdateResources(template, p.PodSpecFlags.Resources.ResourceRequirements, requestsToRemove, limitsToRemove)
 	if err != nil {
 		return err
 	}
 
 	if cmd.Flags().Changed("cmd") {
-		err = servinglib.UpdateContainerCommand(template, p.Command)
+		err = servinglib.UpdateContainerCommand(template, p.PodSpecFlags.Command)
 		if err != nil {
 			return err
 		}
 	}
 
 	if cmd.Flags().Changed("arg") {
-		err = servinglib.UpdateContainerArg(template, p.Arg)
+		err = servinglib.UpdateContainerArg(template, p.PodSpecFlags.Arg)
 		if err != nil {
 			return err
 		}
 	}
 
 	if cmd.Flags().Changed("port") {
-		err = servinglib.UpdateContainerPort(template, p.Port)
+		err = servinglib.UpdateContainerPort(template, p.PodSpecFlags.Port)
 		if err != nil {
 			return err
 		}
 	}
 
-	if cmd.Flags().Changed("min-scale") {
+	// Deprecated "min-scale" in 0.19, updated to "scale-min"
+	if cmd.Flags().Changed("scale-min") || cmd.Flags().Changed("min-scale") {
 		err = servinglib.UpdateMinScale(template, p.MinScale)
 		if err != nil {
 			return err
 		}
 	}
 
-	if cmd.Flags().Changed("max-scale") {
+	// Deprecated "max-scale" in 0.19, updated to "scale-max"
+	if cmd.Flags().Changed("scale-max") || cmd.Flags().Changed("max-scale") {
 		err = servinglib.UpdateMaxScale(template, p.MaxScale)
 		if err != nil {
 			return err
 		}
 	}
 
+	if cmd.Flags().Changed("scale") {
+		if cmd.Flags().Changed("scale-max") {
+			return fmt.Errorf("only --scale or --scale-max can be specified")
+		} else if cmd.Flags().Changed("scale-min") {
+			return fmt.Errorf("only --scale or --scale-min can be specified")
+		} else {
+			err = servinglib.UpdateMaxScale(template, p.Scale)
+			if err != nil {
+				return err
+			}
+			err = servinglib.UpdateMinScale(template, p.Scale)
+			if err != nil {
+				return err
+			}
+		}
+	}
+
 	if cmd.Flags().Changed("autoscale-window") {
 		err = servinglib.UpdateAutoscaleWindow(template, p.AutoscaleWindow)
 		if err != nil {
@@ -429,24 +387,24 @@ func (p *ConfigurationEditFlags) Apply(
 	if cmd.Flags().Changed("label") || cmd.Flags().Changed("label-service") || cmd.Flags().Changed("label-revision") {
 		labelsAllMap, err := util.MapFromArrayAllowingSingles(p.Labels, "=")
 		if err != nil {
-			return errors.Wrap(err, "Invalid --label")
+			return fmt.Errorf("Invalid --label: %w", err)
 		}
 
 		err = p.updateLabels(&service.ObjectMeta, p.LabelsService, labelsAllMap)
 		if err != nil {
-			return errors.Wrap(err, "Invalid --label-service")
+			return fmt.Errorf("Invalid --label-service: %w", err)
 		}
 
 		err = p.updateLabels(&template.ObjectMeta, p.LabelsRevision, labelsAllMap)
 		if err != nil {
-			return errors.Wrap(err, "Invalid --label-revision")
+			return fmt.Errorf("Invalid --label-revision: %w", err)
 		}
 	}
 
 	if cmd.Flags().Changed("annotation") {
 		annotationsMap, err := util.MapFromArrayAllowingSingles(p.Annotations, "=")
 		if err != nil {
-			return errors.Wrap(err, "Invalid --annotation")
+			return fmt.Errorf("Invalid --annotation: %w", err)
 		}
 
 		annotationsToRemove := util.ParseMinusSuffix(annotationsMap)
@@ -457,18 +415,18 @@ func (p *ConfigurationEditFlags) Apply(
 	}
 
 	if cmd.Flags().Changed("service-account") {
-		err = servinglib.UpdateServiceAccountName(template, p.ServiceAccountName)
+		err = servinglib.UpdateServiceAccountName(template, p.PodSpecFlags.ServiceAccountName)
 		if err != nil {
 			return err
 		}
 	}
 
 	if cmd.Flags().Changed("pull-secret") {
-		servinglib.UpdateImagePullSecrets(template, p.ImagePullSecrets)
+		servinglib.UpdateImagePullSecrets(template, p.PodSpecFlags.ImagePullSecrets)
 	}
 
 	if cmd.Flags().Changed("user") {
-		servinglib.UpdateUser(template, p.User)
+		servinglib.UpdateUser(template, p.PodSpecFlags.User)
 	}
 
 	return nil
@@ -477,7 +435,7 @@ func (p *ConfigurationEditFlags) Apply(
 func (p *ConfigurationEditFlags) updateLabels(obj *metav1.ObjectMeta, flagLabels []string, labelsAllMap map[string]string) error {
 	labelFlagMap, err := util.MapFromArrayAllowingSingles(flagLabels, "=")
 	if err != nil {
-		return errors.Wrap(err, "Unable to parse label flags")
+		return fmt.Errorf("Unable to parse label flags: %w", err)
 	}
 	labelsMap := make(util.StringMap)
 	labelsMap.Merge(labelsAllMap)
@@ -488,14 +446,14 @@ func (p *ConfigurationEditFlags) updateLabels(obj *metav1.ObjectMeta, flagLabels
 	return nil
 }
 
-func (p *ConfigurationEditFlags) computeResources(resourceFlags ResourceFlags) (corev1.ResourceList, error) {
+func (p *ConfigurationEditFlags) computeResources(resourceFlags knflags.ResourceFlags) (corev1.ResourceList, error) {
 	resourceList := corev1.ResourceList{}
 
 	if resourceFlags.CPU != "" {
 		cpuQuantity, err := resource.ParseQuantity(resourceFlags.CPU)
 		if err != nil {
 			return corev1.ResourceList{},
-				errors.Wrapf(err, "Error parsing %q", resourceFlags.CPU)
+				fmt.Errorf("Error parsing %q: %w", resourceFlags.CPU, err)
 		}
 
 		resourceList[corev1.ResourceCPU] = cpuQuantity
@@ -505,7 +463,7 @@ func (p *ConfigurationEditFlags) computeResources(resourceFlags ResourceFlags) (
 		memoryQuantity, err := resource.ParseQuantity(resourceFlags.Memory)
 		if err != nil {
 			return corev1.ResourceList{},
-				errors.Wrapf(err, "Error parsing %q", resourceFlags.Memory)
+				fmt.Errorf("Error parsing %q: %w", resourceFlags.Memory, err)
 		}
 
 		resourceList[corev1.ResourceMemory] = memoryQuantity
