@@ -23,6 +23,7 @@ import (
 	"knative.dev/client/pkg/kn/commands"
 	"knative.dev/client/pkg/kn/commands/flags"
 	"knative.dev/client/pkg/sources/v1alpha2"
+	"knative.dev/client/pkg/util"
 )
 
 // NewAPIServerCreateCommand for creating source
@@ -31,11 +32,11 @@ func NewAPIServerCreateCommand(p *commands.KnParams) *cobra.Command {
 	var sinkFlags flags.SinkFlags
 
 	cmd := &cobra.Command{
-		Use:   "create NAME --resource RESOURCE --service-account ACCOUNTNAME --sink SINK --mode MODE",
-		Short: "Create an ApiServer source.",
+		Use:   "create NAME --resource RESOURCE --sink SINK",
+		Short: "Create an api-server source",
 		Example: `
   # Create an ApiServerSource 'k8sevents' which consumes Kubernetes events and sends message to service 'mysvc' as a cloudevent
-  kn source apiserver create k8sevents --resource Event:v1 --service-account myaccountname --sink svc:mysvc`,
+  kn source apiserver create k8sevents --resource Event:v1 --service-account myaccountname --sink ksvc:mysvc`,
 
 		RunE: func(cmd *cobra.Command, args []string) (err error) {
 			if len(args) != 1 {
@@ -62,16 +63,23 @@ func NewAPIServerCreateCommand(p *commands.KnParams) *cobra.Command {
 						"because: %s", name, namespace, err)
 			}
 
-			b := v1alpha2.NewAPIServerSourceBuilder(name).
-				ServiceAccount(updateFlags.ServiceAccountName).
-				EventMode(updateFlags.Mode).
-				Sink(*objectRef)
-
 			resources, err := updateFlags.getAPIServerVersionKindSelector()
 			if err != nil {
 				return err
 			}
-			b.Resources(resources)
+
+			ceOverridesMap, err := util.MapFromArrayAllowingSingles(updateFlags.ceOverrides, "=")
+			if err != nil {
+				return err
+			}
+			ceOverridesToRemove := util.ParseMinusSuffix(ceOverridesMap)
+
+			b := v1alpha2.NewAPIServerSourceBuilder(name).
+				ServiceAccount(updateFlags.ServiceAccountName).
+				EventMode(updateFlags.Mode).
+				Sink(*objectRef).
+				Resources(resources).
+				CloudEventOverrides(ceOverridesMap, ceOverridesToRemove)
 
 			err = apiSourceClient.CreateAPIServerSource(b.Build())
 
